pico-8 cartridge // http://www.pico-8.com
version 5
__lua__

math = {
  max = max,
  min = min,
  floor = flr,
}

_btnstr_data = {}
_btnstr_data.l = 0
_btnstr_data.r = 1
_btnstr_data.u = 2
_btnstr_data.d = 3
_btnstr_data.a = 4
_btnstr_data.b = 5

function printf(string,x,y,width,alignment)
  if alignment == "center" then
    print(string,x+(width-4*#string)/2,y)
  elseif alignment == "left" then
    print(string,x,y)
  elseif alignment == "right" then
    print(string,x+width-4*#string,y)
  else
    printh("invalid alignment")
  end
end

function btnstr(s,p)
  return btn(_btnstr_data[s],p or 0)
end

function btnpstr(s,p)
  return btnp(_btnstr_data[s],p or 0)
end

high_score = 100

debug = {}

function debug.draw()
  cls()
  sspr(20,0,32,32,0,0,128,128)
end

function debug.update()
  if btnpstr("b") then
    change_state(title)
  end
end

splash = {}

function splash.init()
  splash.offset = 64
  splash.wait = 1
end

function splash.draw()
  cls()
  spr(7,28,40-splash.offset,8,4)
  if splash.offset == 0 then
    color(13)
    rectfill(13,48+8*4-1,128-15,48+8+8*4-3)
    color(7)
    printf("missing sentinel software",0,40+8*5,128,"center")
  end
end

function splash.update(dt)
  splash.offset = math.max(0,splash.offset-dt*50)
  if splash.offset == 0 then
    splash.wait -= dt
    if splash.wait <= 0 then
      change_state(title)
    end
  end
end

title = {}

function title.init()
  sfx(0)
  title_option = 1
  title_options = {
    { string = "1 player", action = function()
      player_count = 1
      change_state(game)
    end },
    { string = "2 player", action = function()
      player_count = 2
      change_state(game)
    end },
    { string = "story", action = function()
      change_state(story)
    end },
  }
end

function title.draw()
  cls()
  color(7)
  printf("missing sentinel software",0,64-40,127,"center")
  printf("presents",0,64-32,127,"center")
  spr(3,32+16,32+8,3,4)
  for i,v in pairs(title_options) do
    if i == title_option then
      color(8)
      printf(">"..v.string.."<",0,64+8*i,127,"center")
    else
      color(5)
      printf(v.string,0,64+8*i,127,"center")
    end
  end
  color(6)
  printf("(c) 2015 josefnpat",0,127-8,127,"center")
end

konami = { "u","u","d","d","l","r","l","r","b","a" }
konami_current = 1

function title.update(dt)

  if konami[konami_current] then
    if btnpstr( konami[konami_current] ) then
      konami_current += 1
    end
  else
    konami_current = 1
    change_state(debug)
    cheat_mode = true
  end

  if btnpstr("d") then
    title_option += 1
    if title_option > #title_options then
      title_option = 1
    end
  elseif btnpstr("u") then
    title_option -= 1
    if title_option < 1 then
      title_option = #title_options
    end
  elseif btnstr("a") then
    title_options[title_option].action()
    sfx(1)
  end

end

story = {}

story.data = {
--  "axxxxxxxxxxxxxxxxxxxxxxxxxxxxxa",
  "",
  "",
  "",
  "you are captian red, with her",
  "trusty wingman, captian blue.",
  "the evil alien race, known only",
  "as emacs, are coming to destroy",
  "all of mankind.",
  "",
  "get into the cockpit of your",
  "vi-per-8, and defend the",
  "universe by defeating the six",
  "bosses.",
  "",
  "god speed and good luck.",
}
function story.init()
  cls()
end

function story.draw()
  for i,v in pairs(story.data) do
  printf(v,0,7*(i-1),127,"center")
  end
end

function story.update(dt)
  if btnpstr("b") then
    change_state(title)
  end
end

game = {}

holyshitfactor = 4

function new_player(i)
  local player = {}
  player.x = 32+64*i
  player.y = 128-9
  player.upper_sprite = 0+32*i
  player.lower_sprite = 16+32*i
  player.reload = 0.25/holyshitfactor
  player.reload_dt = 0
  player.bullets = {}
  player.bullet_sprite0 = 1+32*i
  player.bullet_sprite1 = 17+32*i
  player.score = 0
  player.controls = i
  player.speed = 3
  player.overheat_dt = 0
  player.overheat_t = 1

  return player
end

function game.init()

  game.shake = 0

  game.restart = 5

  players = {}

  add(players,new_player(0))
  if player_count >= 2 then
    add(players,new_player(1))
  end

  enemys = {}

  levels = {
    {
      enemyspawn_min = 1.0/player_count,
      boss_sprite = 192,
      boss_hp = 10*player_count*holyshitfactor,
      boss_spawn_dt = 0,
      boss_spawn_t = 32,
    },
    {
      enemyspawn_min = 0.8/player_count,
      boss_sprite = 196,
      boss_hp = 20*player_count*holyshitfactor,
      boss_spawn_dt = 0,
      boss_spawn_t = 32,
    },
    {
      enemyspawn_min = 0.6/player_count,
      boss_sprite = 200,
      boss_hp = 30*player_count*holyshitfactor,
      boss_spawn_dt = 0,
      boss_spawn_t = 32,
    },
    {
      enemyspawn_min = 0.4/player_count,
      boss_sprite = 204,
      boss_hp = 40*player_count*holyshitfactor,
      boss_spawn_dt = 0,
      boss_spawn_t = 32,
    },
    {
      enemyspawn_min = 0.2/player_count,
      boss_sprite = 136,
      boss_hp = 50*player_count*holyshitfactor,
      boss_spawn_dt = 0,
      boss_spawn_t = 32,
    },
    {
      enemyspawn_min = 0.1/player_count,
      boss_sprite = 140,
      boss_hp = 60*player_count*holyshitfactor,
      boss_spawn_dt = 0,
      boss_spawn_t = 32,
    },

  }

  current_level = 1

  set_level(current_level)

  explosions = {}

  flashes = {}

  total_score = 0

end

inbetweenlevels = -2

function set_level(level)
  enemyspawn_t = levels[level].enemyspawn_min;
  enemyspawn_dt = inbetweenlevels
  sfx(2)
end

background_dt = 0

function distance(a,b)
  local lol = sqrt( (a.x-b.x)^2 + (a.y-b.y)^2 )
  if lol < 0 then
    return 32760
  end
  return lol
end

function game.update(dt)

  if game.restart <= 0 then
    change_state(title)
  end

  background_dt += dt * (enemyspawn_dt < 0 and 10 or 1)

  if enemyspawn_dt < 0 then
    camera(rnd()*2-1,rnd()*2-1)
  else
    camera(rnd()*game.shake/4,rnd()*game.shake/4)
  end

  game.shake *= 0.8

  if levels[current_level] then

    enemyspawn_dt += dt

    levels[current_level].boss_spawn_dt += dt

    if levels[current_level].boss_spawn_dt >
      levels[current_level].boss_spawn_t and
      not levels[current_level].boss_entity then

      local boss = {
        x = (128)/2,
        y = -8*4,
        sprite = levels[current_level].boss_sprite,
        speed = 1,
        size = 4,
        boss = true,
        hp = levels[current_level].boss_hp,
        hp_max = levels[current_level].boss_hp,
      }
      levels[current_level].boss_entity = boss
      add(enemys,boss)
      sfx(3)
    end

    if levels[current_level].boss_entity and levels[current_level].boss_entity.hp <= 0 then
      levels[current_level].boss_entity = nil
      current_level += 1
      enemys = {}
      if levels[current_level] then
        set_level(current_level)
      else
        sfx(4)
      end
    end

  end

  for _,player in pairs(players) do

    player.reload_dt += dt

    local speed = player.speed

    if btnstr("a",player.controls) then
      if player.reload_dt > player.reload then
        player.reload_dt = 0
        local bullet = {
          x = player.x,
          y = player.y,
          drift = (rnd()*2-1)*player.overheat_dt/player.overheat_t,
          dt = rnd(),
        }
        add(player.bullets,bullet)
        sfx(5+player.controls)
      end
      speed = 2
      player.overheat_dt = math.min(player.overheat_t,player.overheat_dt+dt)
    else
      player.overheat_dt = math.max(0,player.overheat_dt-dt*10)
    end

    if btnstr("l",player.controls) then player.x-=speed end
    if btnstr("r",player.controls) then player.x+=speed end
    if btnstr("u",player.controls) then player.y-=speed end
    if btnstr("d",player.controls) then player.y+=speed end

    player.x = math.min(127,math.max(0,player.x))
    player.y = math.min(127-8,math.max(63,player.y))

    for _,bullet in pairs(player.bullets) do
      if bullet.y < 0 then
        del(player.bullets,bullet)
      else
        bullet.y-=5
        bullet.x+=bullet.drift*2
        bullet.dt+=dt
      end
    end

  end

  if enemyspawn_dt > enemyspawn_t then
    enemyspawn_dt = 0
    local enemy = {
      x = rnd()*127,
      y = 0,
      sprite = 128+math.floor(rnd()*8),
      speed = rnd()*2+1,
      size = 1,
      hp = holyshitfactor,
      direction = flr(rnd()*3)-1,
    }
    add(enemys,enemy)
  end

  for _,enemy in pairs(enemys) do
    if enemy.boss then
      enemy.y = math.min(32,enemy.y + enemy.speed)
    else
      enemy.y += enemy.speed
      enemy.x += enemy.direction
    end
    if enemy.x >= 128 or enemy.x <= 0 then
      enemy.direction = -enemy.direction
    end
    for _,player in pairs(players) do
      if distance(player,enemy) < 8 and not cheat_mode then
        local explosion = {
          x = player.x,
          y = player.y,
          dt = 0,
        }
        add(explosions,explosion)
        game.shake += 100
        del(players,player)
        add(flashes,{x=player.x,y=player.y,size=2})
        sfx(8)
        if #players <= 0 then
          sfx(10)
        end
        --printh("player hit: "..distance(player,enemy))
      end
    end
    for _,player in pairs(players) do
      for _,bullet in pairs(player.bullets) do
        if distance(bullet,enemy) < 4 + enemy.size*4 then
          local explosion = {
            x = bullet.x,
            y = bullet.y,
            dt = 0,
          }
          add(explosions,explosion)
          game.shake += 1
          del(player.bullets,bullet)
          sfx(7)
          enemy.hp -= 1
          if enemy.hp <= 0 then
            local explosion = {
              x = enemy.x,
              y = enemy.y,
              dt = 0,
            }
            add(explosions,explosion)
            del(enemys,enemy)
            game.shake += enemy.size * 10
            add(flashes,{x=enemy.x,y=enemy.y,size=enemy.size})
            if not levels[current_level].boss_entity or
              enemy == levels[current_level].boss_entity then
              player.score += enemy.size
              total_score += enemy.size
              high_score = math.max(total_score,high_score)
            end
            sfx(9)
            --printh("enemy hit: "..distance(bullet,enemy))
          end
        end
      end
    end
    if enemy.y > 127 then
      del(enemys,enemy)
    end
  end

  for _,explosion in pairs(explosions) do
    explosion.dt += dt*8
    if explosion.dt >= 9 then
      del(explosions,explosion)
    end
  end

  if #players <= 0 then
    game.restart -= dt
  elseif not levels[current_level] then
    game.restart -= dt
  end

end

function game.draw()

  if #players > 0 then
    cls()
  end

  for x = 0,0 do
    for y = -1,3 do
      spr(64,x*8*4,y*8*4+(background_dt*10)%32,16,4)
    end
  end

  for _,player in pairs(players) do
    spr(player.upper_sprite,player.x-8,player.y-8,1,2)
    spr(player.upper_sprite,player.x,player.y-8,1,2,true,false)
  end

  for _,player in pairs(players) do
    for _,bullet in pairs(player.bullets) do
      local bsprite = player.bullet_sprite0
      if bullet.dt*4%1 > 0.5 then
        bsprite = player.bullet_sprite1
      end
      spr(bsprite,bullet.x-4,bullet.y-4)
    end
  end

  if levels[current_level] and levels[current_level].boss_entity then
    local ratio = levels[current_level].boss_entity.hp /
      levels[current_level].boss_entity.hp_max
    color(5)
    rectfill(16,10,127-16,14)
    color(8)
    rectfill(17,11,17+(127-32-2)*ratio,13)
    color(7)
  end

  for _,enemy in pairs(enemys) do
    spr(enemy.sprite,enemy.x-4*enemy.size,enemy.y-4*enemy.size,enemy.size,enemy.size)
  end

  for _,explosion in pairs(explosions) do
    spr(144+flr(explosion.dt),explosion.x-4,explosion.y-4)
  end

  color(10)
  for _,flash in pairs(flashes) do
    circfill(flash.x,flash.y,flash.size*8*2*rnd())
    flash.count = (flash.count or (flash.size+3)) - 1
    if flash.count <= 0 then
      del(flashes,flash)
    end
  end

  color(8)

  printf("score:"..total_score .. " high: "..high_score,0,4,128,"left")
  printf("level "..current_level,0,4,128,"right")
  if #players <= 0 then
    color(0)
    rectfill(0,63,127,63+6)
    color(8)
    printf("mission failure: game over",0,64,128,"center")
  elseif not levels[current_level] then
    color(0)
    rectfill(0,63,127,63+6)
    color(8)
    printf("mission success: you win",0,64,128,"center")
  end

end


function change_state(state)
  state.__initdone = nil
  current_state = state
end

function _init()
  change_state(splash)
end

function _update()

  if not current_state.__initdone then
    if current_state.init then
      current_state.init()
    end
    current_state.__initdone = true
  end

  if current_state.update then
    current_state.update(1/30)
  end
end

function _draw()
  if current_state.__initdone and current_state.draw then
    current_state.draw()
  end
end




















__gfx__
0000000e000880000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000e008888000000000000000000000000000000000000000000000000000000000000000000000000000000000000010000000000000000000000000000
000000e8008888000000000000000000000000000000000000000000000000000000000000000000000000000000000000016000500000000000000000000000
000000e8008888000000000000000000000000000000000000000000000000000000000000000000000000000000000115000511000000000000000000000000
00000e8800888800000000000000000000000b000000000000000000000000000000000000000000000000000000000000700500050000000000000000000000
00000e88008ee80000000000000000000000b3b00000000000000000000000000000000000000000000000000000000500550010151000000000000000000000
0000e88700eeee000000000000000000000b333b00000000000000000000000000000000000000000000000000000000550d00055d0000000000000000000000
0000e87700e00e00000000000000000000b33333b0000000000000000000000000000005000000000000000d000000000d001110110000000000000000000000
000e887a0008800000000000066666666b333666666666000000000000000000000000d710000000000000d75000100000100d00150051100000000000000000
00be88aa0088880000000000677777777633677777777500000000000000000000000066d000000000155066600150d11010055500d050001000000000000000
0be888aa008888000000000067777777753367777777500000000000000000000000006670000000000011dd7005551051110605d01511650100000000000000
b328889a008888000000000005577775533335577775b00000000000000000000000007660d0000010000157700505005d0500d15551d6100100000000000000
333288890088880000000000000677533333333677533b000000000000000000000000d606d10050050006067000516001115070165d51501d00000000000000
53332888008ee800000000000006775333333367753333b0000000000000000000000007655d0000050015d16000d61d001d0011766d0d500110000000000000
0555522200eeee000000000000b67753333336775366666b000000000000000000000005706110000116115d00555765051111d67667dd1d6150010000000000
000009a900e00e00000000000b367753333367753677777500000000000000000000000067d61510101001165057676600dd557777776750d515000000000000
0000000c000dd000000000000026775333367753367333752000000000000000000000055567ddd0006770d55516777655d6d777777766d50500000000000000
0000000c00dddd00000000000006775333676633367333750000000000000000000000061d10055000506d5505167775515dd67777776ddd0501000000000000
000000cd00dddd000000000000067753367765333677777500000000000000000000000d565d06d000005660d10d776d05665776777d56660510100000000000
000000cd00dddd000000000000067753677533333677777500000000000000000000001051d55551011d6d00000566d150150677667666500550000000000000
00000cdd00dddd0000000000000677667753663336732075000000000000000000000051dd677100176d5dd1500d5651015d0d77d7d711d01000000000000000
00000cdd00dccd00000000000006777775336533367200750000000000000000000000d657777000d7777dd11005555000dd007676655d050000000000000000
0000cdd700cccc000000000000067777533365333677777500000000000000000000000d115677577776d50000010d1000dd50d6676650015510000000000000
0000cd7700c00c0000000000000655550233655332555550000000000000000000000000d7501dd651005d6600001100101d500016d1050010000005d66d1000
000cdd7a000dd000000000000000000000233333200000000000000000000000000000000167500105d776510500d5000151011005d15050000005dd111dd510
00bcddaa00dddd00000000000000000000023332000000000000000000000000000000000001677777d100015555511001510100010501000005650000000000
0bcdddaa00dddd0000000000000000000000232000000000000000000000dd56667776dd67d616511000000055000000000000166655111156dd100000000000
b31ddd9a00dddd000000000000000000000002000000000000000000000d76777777777677777ddd000000000000067777777767d5d67777d500000000000000
3331ddd900dddd000000000000000000000000000000000000000000001777777d6611d6d67656666d0000000055015d511d1d11000001000100000000000000
53331ddd00dccd0000000000000000000000000000000000000000000077d6d51d05555551556d11001000000000000000000000000000000000000000000000
0555511100cccc000000000000000000000000000000000000000000056510010000000100000000000000000000000000000000000000000000000000000000
000009a900c00c000000000000000000000000000000000000000000001100000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000060000000000000007000000000000000000000000000000000000000000000000000000000000700000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000070000000000000000000700000000000000000000000000000000000000000000000070000000000
00000006070600000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000060000000000000000000006000000000000000000000000000
00000000060000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000500000000000000000000000000000000000000000000000000000000000000050000000000000000000
00000000000000000000000000000000000000000005650000000000000000000000000060000000000000000000000000000000000565000000000000000000
00000000000000000000000005000000000070000000500000000000000000000000000000000000000000000000000000000000000050000000000000000000
00000000000000000000000057500000000000000000000000000000006000000000006070600000000000000000000000000000000000000000000000000000
00000600000000070000000005000000000000000000000000000000000000000000000000000000000000000700000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000060000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000006000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000070000060000000000000000000000000000000000000000000000070000000000000000000000000000
00070000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000006070600000000000000000000070000000000000000000000000000000000500000000007000000
00000000000700000000000000000000000000000000000000000000000000000000000000000000005000000000000000000000000005650000000000000000
00000000000000000000000000000000000700000000000000060000000000000000000000000000056500000000000000000000000000500000000000000000
00000000000000000000000000000000000000005000000000000000000000000000500000000000005000000000000000000000000000000000000000000000
00000000000000000000000000000600000000056500000000000000000000000005650000000000000000000000000000000000000000000000000000000000
00000000000000000000500000000000000000005000000000000000000700000000500000000000000000000000000000000000070000000000000000000000
00000000000000000005750000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000500000000000000000000000000000000000000000000000000000000000000000000070000000000000000000000000060000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000600000000000000000000000000000000000060000000000000000000000000000000000000000000000000000000000000006070600000700
00000000000000000000000000000000000000000007000000000000000000000000000070000000000000000000000000000000000000000000000000000000
00000000000000000000000000007000000000000000000000000000000000000000000000000000000600000000000000000000000000700000060000000000
00000070000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
22200222000dd0000330033000022000000990000010010006600660044004400000000000511111111111000000000001010101511500000000511510101010
289889820dd99dd03bbbbbb3000220000009900000100100006666000400004000000000555515151515150000000000000515105dd5100000015dd501515000
089999800d9aa9d0300bb00300022000000990000010010000699600045555400000000055d551515151511000000000000115005dd5100000015dd500511000
089aa9800ccaacc000b99b00022ee2204449944401c11c10007aa700004994000000000055655151515151100000000000001000122000000000022100000000
008aa80000c77c0000baab0002e99e20994994991c9999c1007aa700554aa4550000000005565515151511000000000000000020244200000000244202000000
0087780000cccc0000b77b000eeaaee0094aa4900caaaac000777700404774040000000000555511111110000000000000000220244200000000244202200000
0088880000c00c00000bb0000ee77ee00997799000c77c0000777700444444440000000000566d55555550000000000000002220244420000002444202220000
0008800000c00c00000bb00000eeee0000999900000cc00000077000044004400000000000566d55555510000000000000022220244442000024444202222000
00888800008888000044440000555500000000000000000000000000000000000000000000056dd5555500000000000000220520242444222244424202502200
08999980088998800488884005444450005555000005500000000000000000000000000000056dd5555100000000000000205510242444444444424201550200
89977998889999884899998454499445055445500054450000055000000000000000000000005551111000000000000000205d50242444444444424205d50200
897777988997799848979984549889450548845005488450005485000005500000000000000056d5551000000000000000201550242444444444424205510200
89777798899779984899798454988945054884500548845000584500000550000000000000005551110000000000000000205d50242422222222424205d50200
899779988899998848999984544994450554455000544500000550000000000000000000000056d5551000000000000000205510242222222222224201550200
08999980088998800488884005444450005555000005500000000000000000000000000000005551110000000000000000205d50020255144155202005d50200
00888800008888000044440000555500000000000000000000000000000000000000000000005dd555100000000000000020155000025d5445d5200005510200
00000000000000000000000000000000000000000000000000000000000000000000000155001551110000155000000000205d50000215544551200005d50200
000000000000000000000000000000000000000000000000000000000000000000550005550015d5551000155100110000005510000244222244200001550000
00000000000000000000000000000000000000000000000000000000000000000566000220001151110000011001551000005d50000044444444000005d50000
000000000000000000000000000000000000000000000000000000000000000005666000000005d5551000000005551000001550000022222222000005510000
0000000000000000000000000000000000000000000000000000000000000000056666600055511111155600055d551000005d50000004444440000005d50000
00000000000000000000000000000000000000000000000000000000000000000566666655555dddddddd66666dd551000005510000000244200000001550000
00000000000000000000000000000000000000000000000000000000000000005666666655555dddddddd66666dd555100005d50000000444400000005d50000
00000000000000000000000000000000000000000000000000000000000000005666666655555111111dd66666dd555100000100000000244200000000100000
000000000000000000000000000000000000000000000000000000000000000056666666555513333331666666dd5551000000000000004ee400000000000000
00000000000000000000000000000000000000000000000000000000000000000566666655555133331d666666dd551000000000000000244200000000000000
0000000000000000000000000000000000000000000000000000000000000000056666665555551331dd666666d5551000000000000004e33e40000000000000
000000000000000000000000000000000000000000000000000000000000000005666666d55555511ddd666666d55510000000000000046b3640000000000000
0000000000000000000000000000000000000000000000000000000000000000056666666555555ddddd666666d55510000000000000046b3640000000000000
00000000000000000000000000000000000000000000000000000000000000000056666665551115555566666d555100000000000000046b3640000000000000
00000000000000000000000000000000000000000000000000000000000000000056666655110000000055556d55510000000000000004466440000000000000
00000000000000000000000000000000000000000000000000000000000000000005555500000000000000005511100000000000000000444400000000000000
00000000000005555550000000000000000000000111111001111110000000000000000000000000000000000000000000000000000001111000000000000000
0000000000005ffffff5000000000000000000000155555115555510000000000000000000000000000000000000000000000000000013311110000000000000
000000000005ffffffff500000000000000000000155555555555510000000000000000000000000000000000000000000000001110033311111001110000000
00000001005ffff44ffff50010000000000000001155ddd55ddd5511000000000000000000000000000000000000000001111115551133311111011551111110
0000000105ffff4444ffff501000000000000001d155ddd55ddd551d10000000000220000000000000000000000220001dddd555511133111111011555555dd1
000000015fff44444444fff5100000000000001dd155ddd55ddd551dd1000000000020000000000000000000000200001dddd555511133111111011555555551
00000001ffff45544554ffff100000000000001ddd155dddddd551ddd10000000022200000000000000000000002220001ddd555511331111111101155555510
00000005fff45ff4fff54fff50000000005001ddddddddd11ddddddddd1005000022200000000000000000000002220001dddd55111331111111101115555510
00000005f4445554f555444f5000000000511dd5ddd1dd1551dd1ddd5dd1150002422000005151511515150000022420001dddd5113311111111110111555100
0000000555555ff4fff5555550000000001ddd155d5d1d5335d1d5d551ddd10002422200001155d55d551100002224200001ddd1113311111111110111111000
0000000115155554f5555151100000000051ddd5dd5d5d5b35d5d5dd5ddd15000244220000005151151500000022442000001dd1133111111111111011100000
0000000001015ff4fff51010000000000050dddddd5d5d5bb5d5d5dddddd050002442200001151222215110000224420000001d1133111111111111011000000
0000000000005554f55500000000000000505ddddd565d5bb5d565ddddd505002444122000012244442210000221444200000011331111111111111100000000
0000000000005ff4fff50000000000000050505dddd5ddd55ddd5dddd50505002445d1220002444444442000221d544200000001331111100111111100000000
0000000000005554f5550000000000000050505dddddddddddddddddd505050024445422202e44444444e2022245444200000001331133151011111100000000
0000000000055ff4fff55000000000000050001dddd1111111111dddd10005002224444222e4154444514e222444422200000013311331551101111110000000
0000000000055554455550000000000000500055ddd1000110001ddd550005000002224222e1554444551e222422200000000013313315555110111110000000
00000000005ff44444fff500000000000000005dddd1000110001dddd50000000001552022e1554444551e220255100000000013333315511510111110000000
00100000115ff44444fff511000001000000005d1dd1000000001dd1d50000000005dd1000e4154444514e0001dd5000000001333331d5133111011111000000
00001001105ff44444fff50110010000000000551dd1000000001dd15500000000015100002e44433444e20000151000000001333331d5bb3311011111000000
00115115105ff45555fff501511511000000005d1dd1000000001dd1d50000000005d5100002e4b333422000015d500000000133331dd56bb311101111000000
00011111155ff56dd65ff551111110000000005d1dd1000000001dd1d500000000001000000024bb334200000001000000001331111dd56bb311100001100000
0005f444455ff5dddd5ff554444f5000000000551dd1000000001dd15500000000005100000024bb334200000015000000001310001ddd566151100000100000
00005f44455ff5d33d5ff55444f500000000005d1dd1000000001dd1d50000000000000000002e6b36e200000000000000013100001ddd566551100000010000
000005f4455ff5b3335ff5544f5000000000005d1dd1000000001dd1d5000000000000000000026b362000000000000000011000001dddd55551100000000000
0000005f455ff1bb331ff554f5000000000000551d100000000001d1550000000000000000000246642000000000000000000000001ddddd5551100000000000
0000005f445ff1bbb31ff544f50000000000005d1d100000000001d1d500000000000000000002444420000000000000000000000001dddd5551000000000000
0000005ff444440bb044444ff50000000000005d1d100000000001d1d500000000000000000002e44e20000000000000000000000001ddddd511000000000000
0000005fff44440bb04444fff5000000000000551d100000000001d155000000000000000000002ee2000000000000000000000000001dddd110000000000000
00000005fff4444004444fff5000000000000001dd100000000001dd10000000000000000000002ee2000000000000000000000000001dddd110000000000000
0000000055ffff4004ffff550000000000000001dd100000000001dd100000000000000000000022220000000000000000000000000001ddd100000000000000
00000000005555555555550000000000000000011110000000000111100000000000000000000020020000000000000000000000000000111000000000000000
__gff__
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
__map__
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
__sfx__
000a000014570165701a570205702457027570235701f570185700000015570000001557000000155700000019570000001f570000000000026570000002e570345702c57027570215701e5701f5702157025570
0001000014170151701717018170191701b1701c1701c1701d1701f1702017020170201701f1701e1701e1701d1701b1701917019170171700000015170141701317011170000000e170000000b1700a17000000
000300000d3700e3701037011370133701537016370193701a3701e37022370243700b3700d3700f37011370123701437017370193701c3701e37022370293702f3700f370133701937021370283702f37038370
000300001e7701e7701f77021770237702677026770277702777026770247702277021770207701e7701d7701d7701c7701c7701c7701b7701c7701d7701e7702077021770247702677027770287702977000000
001000000b3700c3700d3700f37012370153700d3700d370103701237014370163701a3701e37021370143701137014370163701a3701d37023370253702e370353702837027370293702b3702f3703337038370
000100001111015110181101a1101b1101c1101d1101d1101e1101e1101f1101f1101e1101d1101b1101a11019110181101711016110141101211011110101100f1100e1100d1100d1100d1100d1100d1100d110
000100001711018110191101a1101c1101d1101d1101d1101d1101c1101c1101b1101a1101911016110131101111011110101100f1100f110111101311015110171101811018110181101611013110101100e110
00010000136501465014640146401463014620136201261012610106100e6100c61009610086100661004610026100261002600016000460004600036000360013600116000f6000d6000b600086000360001600
000c00001577000000157001477015700000001377015700000000000014770000001a700147700000000000000001c70014770000001c700000000f7701c700000000a7700a7700a77000000187000000016700
0001000010610116101161010610106100d6100b61008610056100461003610026100161000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00100000236300163002630026302363023630016302363023630026300263023630236302363023630226300263023630236302363023630016302363023630236302263021630216301f6201b6101961015600
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000101c075010701c07001070180701c000010701707001070170701c0001c070010701c000020701c0001c0001c0001c0001c0001c0001c0001c0001c0001c0001c0001c0001c0001c0001c0001c0001c000
001000101c600036001c675016730000001600000001c675016730000000000000000000001673000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
00100000213002830027300273002a300000000000000000000001d3001c300233001d30020300223000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
001000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
__music__
03 10115244
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344
00 41424344

